package liviu.tudor.convertor.provider;

import java.io.IOException;

import javax.microedition.lcdui.Image;

public abstract class ConvertProviderAdapter implements ConvertProvider
{
    /**
     * This will hold all of our field names.
     * For each such field there is a corresponding value in {@link #fieldValues fieldValues}.
     * There is only one field that is a reference field and its index is stored in {@link #referenceField referenceField}.
     * 
     * @see #referenceField
     * @see #fieldValues
     */
    protected String fieldNames[] = null;
    
    /**
     * This array holds the conversion values relative to the reference field.
     * 
     * @see #fieldNames
     * @see #referenceField
     */
    protected double fieldValues[] = null;
    
    /**
     * This holds the index of the reference field.
     * 
     * @see #fieldNames
     * @see #fieldValues
     */
    protected int referenceField = -1;
    
    /**
     * This value stores the computed values after each call to {@link #setFieldValue(int, double) setFieldValue()}
     */
    protected double computedValues[];
    
    /**
     * Shortcut function for {@link #addField(String, double, boolean) addField(name,value,false)}.
     * 
     * @param name Name of the field
     * @param value Value relative to the reference field.
     */
    protected void addField( String name, double value )
    {
        addField( name, value, false );
    }
    
    /**
     * Function to be used by subclasses to allow them to build up the arrays of field values and names.
     * 
     * @param name Name of the field
     * @param value Value relative to the reference field. If this is the reference field then the value 
     * will be set to 1 (one) regardless of the value passed
     * @param isReference Flag to indicate whether this is the reference field or not
     */
    protected void addField( String name, double value, boolean isReference )
    {
        if( isReference )
            value = 1.0;
        if( fieldNames == null )    // first entry?
        {
            fieldNames = new String[1];
            fieldNames[0] = name;
            fieldValues = new double[1];
            fieldValues[0] = value;
        }
        else
        {
            // copy names
            String[] tempNames = new String[fieldNames.length + 1];
            System.arraycopy( fieldNames, 0, tempNames, 0, fieldNames.length );
            fieldNames = tempNames;
            fieldNames[fieldNames.length - 1] = name;
            tempNames = null;
            // copy values
            double[] tempValues = new double[fieldNames.length];   // fieldNames.length has already been updated!
            System.arraycopy( fieldValues, 0, tempValues, 0, fieldNames.length - 1 );
            fieldValues = tempValues;
            fieldValues[fieldValues.length - 1] = value;
            tempValues = null;
        }
        if( isReference )
            referenceField = fieldNames.length - 1;
    }
    
    /**
     * Returns all the names of the fields for which we can provide conversions.
     * 
     * @return List of all the input field names
     * @see #fieldNames
     */
    public String[] getFieldNames()
    {
        return fieldNames;
    }
    
    /**
     * Function to be overriden by subclases which want to specified an icon (image)
     * to be shown in the menu.
     * 
     * @return Path to an image.
     * @see #getImage()
     */
    public abstract String getImageName();
    
    /**
     * Wrapper function -- subclasses should override {@link #getImageName() getImageName()}.
     * This function will try to load the image specified by the {@link #getImageName() getImageName()} and if
     * any errors occur it will return null.
     */
    public Image getImage() 
    {
        try 
        {
            return Image.createImage( getImageName() );
        } 
        catch (IOException e) 
        {
            return null;
        }
    }
    
    /**
     * Function to be overriden in subclasses to check whether the data entered by the user is correct.
     * 
     * @param index Index of the field the user has entered
     * @param value Data entered by the user
     * @throws IllegalArgumentException if data input is not an accepted value
     */
    protected abstract void checkValue( int index, double value ) throws IllegalArgumentException;
    
    /**
     * Returns all the values of the fields, following the conversion made after the user has called
     * {@link #setFieldValue(int, double) setFieldValue()}. 
     * 
     * @return List of all the converted values
     * @see #computedValues
     */
    public double[] getFieldValues()
    {
        return computedValues;
    }
    
    /**
     * This function gets called when the user has input a value in one of the text fields.
     * This function calls {@link #checkValue(int, double) checkValue()} in the subclass to
     * verify the input data is correct and then makes the computation based on the values
     * stored in {@link #fieldValues fieldValues} and stores the result in {@link #computedValues computedValues}
     * and returns it.
     * 
     * @param index Index of the field the user has edited
     * @param value Value the user has input
     * @return List of all the converted values
     * @throws IllegalArgumentException if the value passed is not acceptable
     * @see #computedValues
     * @see #fieldValues
     */
    public double[] setFieldValue( int index, double value ) throws IllegalArgumentException
    {
        checkValue( index, value );
        
        /*
         * Go ahead and compute the values and store them in computedValues
         * Formula is : computedValues[i] = (fieldValues[index] / fieldValues[i]) * value 
         */
        if( fieldValues == null )
            return null;
        computedValues = new double[fieldValues.length];
        for( int i = 0; i < fieldValues.length; i++ )
            computedValues[i] = (fieldValues[index] / fieldValues[i]) * value;
        return computedValues;
    }
}
