package liviu.tudor.convertor.provider;

import java.io.IOException;

import javax.microedition.lcdui.Image;

/**
 * This class converts temperatures.
 * Because this class is more complex, 
 * 
 * @author Liviu Tudor
 */
public class TemperatureConvertor implements ConvertProvider
{
    /**
     * Supported scales
     */
    private String[] fieldNames = new String[] { "Celsius", "Fahrenheit", "Kelvin", "Reaumur", "Rankine" };
    
    /**
     * Stores values computed at last conversion
     */
    private double computedValues[];
    
    /**
     * The computation matrix.
     * Position (i,j) gives the Computation interface to convert from fieldNames[i] to fieldNames[j]
     * Position (i,i) is ignored and it is recommended to remain null
     */
    private Computation computationMatrix[][];
    
    /**
     * Constructor.
     * Here we initialize all of our values we can convert.
     * The reference value is square meter.
     */
    public TemperatureConvertor() 
    {
        // celsius -> fahrenheit
        final Computation celsiusToFahrenheit = new Computation() 
        {
            public double compute( double value )
            {
                return (9.0 / 5.0 ) * value + 32.0;
            }
        };
        
        // fahrenheit -> celsius
        final Computation fahrenheitToCelsius = new Computation() 
        {
            public double compute( double value )
            {
                return (5.0 / 9.0) * ( value - 32.0 );
            }
        };
        
        // celsius -> kelvin
        final Computation celsiusToKelvin = new Computation() 
        {
            public double compute( double value )
            {
                double d = value + 273.15;
                //little trick here -- there is no temperature below 0 Kelvin!!!
                if( d < 0.0 )
                    d = 0.0;
                return d;
            }
        };
        
        // kelvin -> celsius
        final Computation kelvinToCelsius = new Computation() 
        {
            public double compute( double value )
            {
                return (value - 273.15);
            }
        };
        
        // celsius -> Reaumur
        final Computation celsiusToReaumur = new Computation()
        {
            public double compute( double value )
            {
                return (5.0 / 4.0) * value;
            }
        };
        
        // reaumur -> celsius
        final Computation reaumurToCelsius = new Computation()
        {
            public double compute( double value )
            {
                return (4.0 / 5.0) * value;
            }
        };
        
        // rankine -> kelvin
        final Computation kelvinToRankine = new Computation()
        {
            public double compute( double value )
            {
                return 1.80 * value;
            }
        };
        
        // kelvin -> rankine
        final Computation rankineToKelvin = new Computation()
        {
            public double compute( double value )
            {
                return value / 1.80;
            }
        };
        
        
        // fahrenheit -> kelvin
        final Computation fahrenheitToKelvin = new Computation() 
        {
            public double compute( double value )
            {
                return celsiusToKelvin.compute( fahrenheitToCelsius.compute(value) );
            }
        };
        
        // kelvin -> fahrenheit
        final Computation kelvinToFahrenheit = new Computation() 
        {
            public double compute( double value )
            {
                return celsiusToFahrenheit.compute( kelvinToCelsius.compute(value) );
            }
        };
        
        // reaumur -> kelvin
        final Computation reaumurToKelvin = new Computation()
        {
            public double compute( double value )
            {
                return celsiusToKelvin.compute( reaumurToCelsius.compute(value) );
            }
        };
        
        // kelvin -> reaumur
        final Computation kelvinToReaumur = new Computation() 
        {
            public double compute( double value )
            {
                return celsiusToReaumur.compute( kelvinToCelsius.compute(value) );
            }
        };
        
        // reaumur -> fahrenheit
        final Computation reaumurToFahrenheit = new Computation()
        {
            public double compute( double value )
            {
                return celsiusToFahrenheit.compute( reaumurToCelsius.compute(value) );
            }
        };
        
        // fahrenheit -> reaumur
        final Computation fahrenheitToReaumur = new Computation() 
        {
            public double compute( double value )
            {
                return celsiusToReaumur.compute( fahrenheitToCelsius.compute(value) );
            }
        };
        
        // rankine -> celsius
        final Computation rankineToCelsius = new Computation() 
        {
            public double compute( double value )
            {
                return kelvinToCelsius.compute( rankineToKelvin.compute(value) );
            }
        };
        
        // celsius -> rankine
        final Computation celsiusToRankine = new Computation() 
        {
            public double compute( double value )
            {
                return kelvinToRankine.compute( celsiusToKelvin.compute(value) );
            }
        };
        
        // rankine -> fahrenheit
        final Computation rankineToFahrenheit = new Computation() 
        {
            public double compute( double value )
            {
                return kelvinToFahrenheit.compute( rankineToKelvin.compute(value) );
            }
        };
        
        // fahrenheit -> rankine
        final Computation fahrenheitToRankine = new Computation() 
        {
            public double compute( double value )
            {
                return kelvinToRankine.compute( fahrenheitToKelvin.compute(value) );
            }
        };
        
        // rankine -> reaumur
        final Computation rankineToReaumur = new Computation() 
        {
            public double compute( double value )
            {
                return kelvinToReaumur.compute( rankineToKelvin.compute(value) );
            }
        };
        
        // reaumur -> rankine
        final Computation reaumurToRankine = new Computation() 
        {
            public double compute( double value )
            {
                return kelvinToRankine.compute( fahrenheitToReaumur.compute(value) );
            }
        };

        
        /*
         * Finally initialize the computation matrix
         */
        computationMatrix = new Computation[][] { 
                {null, celsiusToFahrenheit, celsiusToKelvin, celsiusToReaumur, celsiusToRankine}, 
                {fahrenheitToCelsius, null, fahrenheitToKelvin, fahrenheitToReaumur, fahrenheitToRankine}, 
                {kelvinToCelsius, kelvinToFahrenheit, null, kelvinToReaumur, kelvinToRankine},
                {reaumurToCelsius, reaumurToFahrenheit, reaumurToKelvin, null, reaumurToRankine},
                {rankineToCelsius, rankineToFahrenheit, rankineToKelvin, rankineToReaumur, null}
        };
        computedValues = new double[fieldNames.length];
    }

    /**
     * Returns all the names of the fields for which we can provide conversions.
     * 
     * @return List of all the input field names
     * @see #fieldNames
     */
    public String[] getFieldNames() 
    {
        return fieldNames;
    }

    /**
     * Returns all the values of the fields, following the conversion made after the user has called
     * {@link #setFieldValue(int, double) setFieldValue()}. 
     * 
     * @return List of all the converted values
     * @see #computedValues
     */
    public double[] getFieldValues() 
    {
        return computedValues;
    }

    /**
     * Return our convertor image -- in this case, a thermometer
     */
    public Image getImage() 
    {
        try 
        {
            return Image.createImage( "/img/thermometer.png" );
        } 
        catch (IOException e) 
        {
            return null;
        }
    }

    public String getInfo() 
    {
        return "Provides temperature conversions to and from: Celsius, Kelvin and Fahrenheit";
    }

    /**
     * Name this convertor <i>Weight</i>
     */
    public String getName() 
    {
        return "Temperature";
    }

    /**
     * This function gets called when the user has input a value in one of the text fields.
     * This function makes the computation based on the values stored in {@link #fieldValues fieldValues} 
     * and stores the result in {@link #computedValues computedValues} and returns it.
     * 
     * @param index Index of the field the user has edited
     * @param value Value the user has input
     * @return List of all the converted values
     * @see #computedValues
     * @see #fieldValues
     */
    public double[] setFieldValue( int index, double value ) 
    {
        computedValues[index] = value;
        for( int i = 0; i < fieldNames.length; i++ )
            if( i != index )
                computedValues[i] = computationMatrix[index][i].compute( value );
        return computedValues;
    }
    
    interface Computation
    {
        public double compute( double value );
    }
}
